package org.sevensource.commons.configuration.jdbc.impl.spring;

import javax.mail.Session;
import javax.naming.NameNotFoundException;
import javax.naming.NamingException;

import org.sevensource.commons.configuration.jdbc.impl.DefaultConstants;
import org.sevensource.commons.configuration.jdbc.impl.DefaultEnvironment;
import org.sevensource.commons.configuration.jdbc.spring.support.ApplicationContextProvider;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.config.BeanFactoryPostProcessor;
import org.springframework.beans.factory.config.ConfigurableListableBeanFactory;
import org.springframework.context.ApplicationContextInitializer;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.core.env.ConfigurableEnvironment;
import org.springframework.jndi.JndiLocatorDelegate;
import org.springframework.jndi.JndiObjectFactoryBean;


public class DefaultConfigurationContextInitializer implements ApplicationContextInitializer<ConfigurableApplicationContext> {
	
	private final static Logger logger = LoggerFactory.getLogger(DefaultConfigurationContextInitializer.class);
	
	private final static String APPLICATION_CONTEXT_PROVIDER_BEAN_NAME = ApplicationContextProvider.class.getSimpleName();
	
	
	public void initialize(final ConfigurableApplicationContext applicationContext) {
		final ConfigurableEnvironment env = applicationContext.getEnvironment();
		DefaultEnvironment profileEnvironment = DefaultEnvironment.resolveEnvironment(env);
		
		// add the profile
		if (logger.isInfoEnabled()) {
			logger.info("Adding {} to active Spring profiles", profileEnvironment.name());
		}
		env.addActiveProfile( profileEnvironment.name() );
		
		
		
		applicationContext.addBeanFactoryPostProcessor(new BeanFactoryPostProcessor() {

			@Override
			public void postProcessBeanFactory(
					ConfigurableListableBeanFactory beanFactory)
					throws BeansException {

				if(! beanFactory.containsBeanDefinition(APPLICATION_CONTEXT_PROVIDER_BEAN_NAME)) {
					
					if (logger.isInfoEnabled()) {
						logger.info("Registering {} to statically access the Spring Context", ApplicationContextProvider.class.getName());
					}
				
					ApplicationContextProvider contextProvider = new ApplicationContextProvider();
					contextProvider.setApplicationContext(applicationContext);
					beanFactory.registerSingleton(APPLICATION_CONTEXT_PROVIDER_BEAN_NAME, contextProvider);					
				}
			}
			
		});
					
		//add JNDI Mail Session
		applicationContext.addBeanFactoryPostProcessor(new BeanFactoryPostProcessor() {
			@Override
			public void postProcessBeanFactory(
					ConfigurableListableBeanFactory beanFactory) throws BeansException {
	
				if(JndiLocatorDelegate.isDefaultJndiEnvironmentAvailable()) {
					if (logger.isInfoEnabled()) {
						logger.info("JNDI is available. Proceeding with JNDI lookups");
					}
				
				
					if (logger.isDebugEnabled()) {
						logger.debug("Registering mailSession in BeanFactoryPostProcessor");
					}
		
					final String jndiName = DefaultConstants.MAIL_SESSION;
				
					try {
						final JndiObjectFactoryBean mailFactory = new JndiObjectFactoryBean();
						mailFactory.setCache(true);
						mailFactory.setResourceRef(true);
						mailFactory.setExpectedType(Session.class);
						mailFactory.setJndiName(jndiName);
						mailFactory.afterPropertiesSet();
			
						beanFactory.registerSingleton("mailSession", mailFactory);
					} catch (IllegalArgumentException e) {
						logger.error("Error looking up JNDI MailSession", e);
					} catch(NameNotFoundException nfe) {
						logger.warn("No MailSession registered in JNDI under name [{}]. Skipping mail registration", jndiName);
					}
					catch (NamingException e) {
						logger.error("JNDI Error", e);
					}
				}
			}
		});
	}
}
